"""Routines supporting the interface with a relational database for dynamic sublists. """

import psycopg2 as pgdb
from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
import sys
import copy
import re
import os
import types
import string
import email.Utils
from Mailman.Logging.Syslog import syslog
from Mailman import Errors
from Mailman import ErrorsDlist
from Mailman import mm_cfg
from Mailman.Queue.sbcache import get_switchboard
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from Mailman import Message
from Mailman.ErrorsDlist import *
import urllib

lousyThreadNames = ["thread", "subscribe", "unsubscribe", "sub", "unsub", "help", "join", "owner", "admin", 'mailto', 'http', 'bounces', 'leave', 'request']
    
def getConn(mlist):
    """Return a connection to the database for a given mailing list.  The argument may be either the mlist object or a string."""
    if not mlist:
        return pgdb.connect(host='localhost', database='postgres',
            user='mailman', password='mailman')
    try:
        name = mlist.internal_name()
    except:
        name = mlist.lower()
    return pgdb.connect(host='localhost', database=name,
        user='mailman', password='mailman')

def executeSQL(mlist, commands, ignoreErrors=0):
    """Execute a sequence of SQL commands, returning the result of last one."""
    # In case it's called with a string
    if (type(commands) == types.StringType):
        commands = [commands]
    conn = getConn(mlist)
    cursor = conn.cursor()
    for command in commands:
        syslog('info', "DlistUtils: Making SQL query '%s'", command)
        if ignoreErrors:
            try:
                cursor.execute(command)
            except:
                pass
        else:
            cursor.execute(command)
        syslog('info', "DlistUtils: query succeeded")
    # Return result of final operation
    try:
        result = cursor.fetchone()[0]
    except:
        result = None
        syslog('info', 'Result of query is /%s/', result)
    conn.commit()
    return result

def executeSQLmemberQuery(mlist, command):
    """Execute a SELECT statement, returning the email addresses of matching subscribers."""
    #syslog('info', 'About to execute: %s', command)
    conn = getConn(mlist)
    cursor = conn.cursor()
    cursor.execute(command)
    returnList = []
    for e in cursor.fetchall():
        try:
            newMember = mlist.getMemberCPAddress(e[0])
            returnList.append(newMember)
        except:
            #syslog('error', 'Unable to find user ' + e[0] + ' in internal database for %s', mlist.internal_name());
            syslog('error', 'User name was found through this query: ' + command);
    syslog('info', '   Result is: %s', returnList)
    return returnList

def executeSQLreturningList(mlist, command):
    """Execute a SELECT statement, returning the results as a list."""
    #syslog('info', 'About to execute: %s', command)
    conn = getConn(mlist)
    cursor = conn.cursor()
    cursor.execute(command)
    returnList = []
    for e in cursor.fetchall():
        returnList.append(e[0])
    return returnList

def create_database(name):
    """
    Create a database. This requires us to not be in a transaction.
    """
    conn = getConn(None)
    old_iso_lvl = conn.isolation_level
    conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
    cursor = conn.cursor()
    cursor.execute('create database "%s"' % name)
    conn.set_isolation_level(old_iso_lvl)

def create_dlist(mlist):
    """
    Set the dynamic sublist options for a mailing list and create
    the corresponding postgres database and tables.
    """
    create_database(mlist.internal_name())
    executeSQL(mlist, 
               ["CREATE TABLE subscriber (subscriber_id SERIAL PRIMARY KEY,\
                                          mailman_key VARCHAR(255) UNIQUE,\
                                          preference INT2 DEFAULT 1,\
					  format INT2 DEFAULT 3, \
                                          deleted BOOLEAN DEFAULT FALSE, \
                                          suppress INT2 DEFAULT 0)",
                "CREATE UNIQUE INDEX subscriber_mailman_key ON subscriber(mailman_key)",
                "CREATE TABLE message (message_id SERIAL PRIMARY KEY,\
                                       sender_id INTEGER REFERENCES subscriber,\
                                       subject VARCHAR(255),\
                                       thread_id INTEGER DEFAULT NULL)",
                "CREATE TABLE thread (thread_id SERIAL PRIMARY KEY,\
                                      thread_name CHAR(16),\
                                      base_message_id INTEGER REFERENCES message,\
                                      status INT2 DEFAULT 0,\
                                      parent INTEGER DEFAULT NULL)",
		"CREATE TABLE alias (pseudonym VARCHAR(255) PRIMARY KEY, \
				     subscriber_id INTEGER REFERENCES subscriber)",
                "CREATE TABLE override (subscriber_id INTEGER REFERENCES subscriber,\
                                        thread_id INTEGER NOT NULL REFERENCES thread,\
                                        preference INT2 NOT NULL)",
                # Needed in case non-subscribers post
                "INSERT INTO subscriber (subscriber_id, suppress) VALUES (0,1)"],
               ignoreErrors=1)
    alreadyLocked = mlist.Locked()
    if alreadyLocked == 0:
        mlist.Lock(timeout=mm_cfg.LIST_LOCK_TIMEOUT)
    mlist.dlists = 1
    mlist.require_explicit_destination = 0
    mlist.include_list_post_header = 0
    mlist.include_rfc2369_headers = 0
    mlist.loose_email_matching = 1
    mlist.obscure_addresses = 0

    mlist.Save()
    if alreadyLocked == 0:
        mlist.Unlock()

def getSubscriber_id_raw(mlist, addr):
    if addr == None:
#       syslog('info', 'getSubscriber_id_raw called with addr=None')
        return None
    return executeSQL(mlist,
                      ["SELECT subscriber_id \
                      FROM subscriber \
                      WHERE mailman_key = '%s'" % addr.lower()])

def getSubscriber_id_raw_or_die(mlist, addr):
    result = getSubscriber_id_raw(mlist, addr)
    if (result == None):
        #syslog('error', 'getSubscriber_id_raw_or_die unable to find address /%s/ for mailing list /%s/', addr, mlist.internal_name())
        raise ErrorsDlist.InternalError
    else:
        return result
    
def getSubscriber_id(mlist, msg, msgdata, safe=0, loose=0):
    """Returns the subscriber_id of the sender of a message and sets the 'subscriber_id' field in the msg object."""
    fromAddr = email.Utils.parseaddr(msg['From'])[1]
    try:
        subscriber_id = msgdata['subscriber_id']
        return subscriber_id
    except:
        subscriber_id = getSubscriber_id_raw(mlist, fromAddr)
        if subscriber_id == None:
            bestAddr = canonicalize_sender(mlist, msg.get_senders())
            subscriber_id = getSubscriber_id_raw(mlist, bestAddr)
            if subscriber_id == None:
#               syslog('info', "DlistUtils.getSubscriber_id: subscriber_id is None and safe is %d", safe)
                if safe:
                    # This could happen if a non-member is given permission to post
                    return 0
                else:
                    #syslog('info', "Raising ErrorsDlist.NotMemberError")
                    raise ErrorsDlist.NotMemberError("Your request could not be processed because %s is not subscribed to %s.  Perhaps you are subscribed with a different email address, which forwards to %s.  If so, please log into %s and add this as an 'other incoming email address'." % (fromAddr, mlist.internal_name(), fromAddr, mlist.GetScriptURL('options', 1))) 
        syslog('info', 'subscriber_id = %d', subscriber_id)
        msgdata['subscriber_id'] = subscriber_id
        return subscriber_id

def createMessage(mlist, msg, msgdata):
    """Create a new message (in the database), returning its id."""
    senderID = getSubscriber_id(mlist, msg, msgdata, safe=1, loose=1)
    # extract subject and escape quotes
    subject = msg['Subject'].encode().replace("'", "''")

    try:
        threadID = msgdata['thread_id']
    except:
        threadID = -1
    messageID = executeSQL(mlist,
       [("""INSERT INTO message (sender_id, thread_id, subject)
         VALUES (%d, %d, '%s')""" % (senderID, threadID, subject)),
         """SELECT MAX(message_id) FROM message"""])
    return messageID
                          
def createThread(mlist, msg, msgdata, threadBase):
    """Create a new thread, returning its unique id, name."""
    msgdata['message_id'] = createMessage(mlist, msg, msgdata)
    senderID = getSubscriber_id(mlist, msg, msgdata, safe=1, loose=1)

    threadID = executeSQL(mlist,
                          # These must be atomic
                          ["INSERT INTO thread (base_message_id) VALUES (%d)" % msgdata['message_id'],
                           "SELECT MAX(thread_id) FROM thread"])

    executeSQL(mlist,
               ["UPDATE message SET thread_id=%d WHERE message_id=%d"
                % (threadID, msgdata['message_id'])])

    ## Choose a unique name for the thread

    # Try to get the name from the to-line, e.g., listname+new+name@hostname
    if threadBase:
        threadBase = alphanumericOnly(threadBase).lower()
        if threadBase in lousyThreadNames or not len(threadBase):
            threadBase = None

    # If a name wasn't explicitly specified, try to get one from the subject
    if not threadBase:
        # No thread name was specified -- try to get from subject line.
        # If none in subject line, the threadID will be returned
        threadBase = subjectToName(msg['Subject'].encode(), threadID)

    # Make sure the thread can fit in the field (char(16))
    threadBase = threadBase[:13]
        
    # If the threadBase is not unique, make threadBase unique by appending a number
    num = executeSQL(mlist, 
                     "SELECT COUNT(*) FROM thread WHERE thread_name LIKE '%s%%'"
                     % threadBase)
    if not num:
        threadName = threadBase
    else:
        threadName = threadBase + str(num+1)

    executeSQL(mlist,
               ["UPDATE thread SET thread_name='%s' WHERE thread_id='%d'"
                % (threadName, threadID)])

    return (threadID, threadName)

def alphanumericOnly(s):
    """Filter any non-letter characters from a string"""
    result = [letter for letter in s if letter in string.ascii_letters or letter in string.digits]
    return string.join(result, '')
        
def subjectToName(subject, threadID):
    """Return a lower-case name for a new thread based on the subject, if present, or on the threadID"""
    result = None
    
    if subject == '':
        return str(threadID)

    subjectWords = [alphanumericOnly(w) for w in subject.split()]
    
    # Choose the longest word of 4 or more characters
    maxLength = 3
    maxWord = None
    for word in subjectWords:
        if len(word) > maxLength:
            maxWord = word
            maxLength = len(word)
    if maxWord:
        result = maxWord.lower()

    if not result:
        # Choose the first word that's not a stop word or lousy thread name
        stopWords = ["a", "an", "the", "of", "re",
            "you", "i", "no", "not", "do", "for"]
        for word in subjectWords:
            if word not in stopWords and word not in lousyThreadNames:
                result = word.lower()
                break

    if not result:
        # If no other candidate, just return the first subject word
        result = subjectWords[0].lower()

    return result[:12]

def newThread(mlist, msg, msgdata, threadBase=None):
    """Starts a new thread, including creating and enqueueing the
       initial messages"""
    id, name = createThread(mlist, msg, msgdata, threadBase)
    msgdata['thread_id'] = id
    msgdata['thread_name'] = name

    # Delete any other 'To' headings
    del msg['To']
    msg['To'] =  '%s+%s@%s' % (mlist.internal_name(),
                             name,
                             mlist.host_name)

    # different emails for different prefs, so we need to queue separately
    specialize_and_inject(mlist, msg, msgdata, new_thread=True, pref=True)
    specialize_and_inject(mlist, msg, msgdata, new_thread=True, pref=False)
    # Make original message go to nobody (but be archived)
    msgdata['recips'] = []

def continueThread(mlist, msg, msgdata, threadReference):
    """Continue an existing thread, no return value."""

    senderID = getSubscriber_id(mlist, msg, msgdata, safe=1, loose=1)
    msgdata['message_id'] = createMessage(mlist, msg, msgdata)

    specialize_and_inject(mlist, msg, msgdata, new_thread=False, pref=True)
    # Make original message go to nobody (but be archived)
    msgdata['recips'] = []

continue_sql = """SELECT mailman_key
                  FROM subscriber
                  WHERE deleted = FALSE
                        AND suppress = 0
                        AND
                        (
                            (subscriber.preference = 1
                                AND NOT EXISTS 
                                (SELECT * FROM override
                                 WHERE override.subscriber_id =
                                         subscriber.subscriber_id
                                     AND override.thread_id = %(thread_id)d
                                     AND override.preference = 0)
                            )
                            OR
                            (subscriber.preference = 0
                                AND EXISTS
                                (SELECT * FROM override
                                 WHERE override.subscriber_id =
                                         subscriber.subscriber_id
                                     AND override.thread_id = %(thread_id)d
                                     AND override.preference = 1)
                            )
                        )"""

new_sql = """SELECT mailman_key
             FROM subscriber
             WHERE preference=%(pref)d
                   AND deleted=FALSE
                   AND suppress=0"""

def specialize_and_inject(mlist, msg, msgdata, new_thread, pref):
    sql_params = dict(thread_id=msgdata["thread_id"], pref=pref)
    if new_thread:
        # email everyone
        sql = new_sql
    else:
        # email selected people
        sql = continue_sql

    setFooterText(mlist, msg, msgdata, pref)

    dq = get_switchboard(mm_cfg.DLISTQUEUE_DIR)

    msgdata['recips'] = executeSQLmemberQuery(mlist, sql % sql_params)
    dq.enqueue(msg, msgdata, listname=mlist.internal_name())

def threadIDandName(mlist, threadReference):
    """Given thread_id or thread_name, determine the other, returning (thread_id, thread_name)"""
    try:
        thread_id = int(thread_reference)
        try:
            thread_name = executeSQL(mlist,
                                     ["SELECT thread_name FROM thread WHERE thread_id = %d" % thread_id])
        except:
            raise NonexistentThreadRequest("Your message could not be sent because you specified a nonexistent conversation (%d).  Perhaps you meant to start a new conversation, which you can do by addressing your message to %s+new@%s" % (thread_id, mlist.real_name, mlist.host_name))
    except:
        thread_name = threadReference.lower()
	syslog('info', 'DlistUtils: thread_name = /%s/', thread_name)
        thread_id = executeSQL(mlist,
                               ["SELECT thread_id FROM thread WHERE thread_name = '%s'" % thread_name])
        if thread_id == None:
            raise NonexistentThreadRequest("Your message could not be sent because you addressed it to a nonexistent conversation (%s).  Perhaps you meant to start a new conversation named %s, which you can do by addressing your message to %s+new+%s@%s" % (thread_name, thread_name, mlist.real_name, thread_name, mlist.host_name))
    return (thread_id, thread_name)
    
def subscribeToThread(mlist, msg, msgdata):
    """Subscribe the sender of a message to a given thread."""
    override(mlist, msg, msgdata, 1)

def unsubscribeFromThread(mlist, msg, msgdata):
    """Unubscribe the sender of a message from a given thread."""
    override(mlist, msg, msgdata, 0)
    
def override(mlist, msg, msgdata, preference):
    """Subscribe or unsubscribe a user from a given thread."""
    #syslog('info', 'DlistUtils: in override')
    subscriberID = getSubscriber_id(mlist, msg, msgdata, loose=1)
    threadID = msgdata['thread_id']
    _override_inner(mlist, subscriberID, threadID, preference)

def override_from_web(mlist, subscriberID, thread_reference, preference):
    """Add an override entry to the database, returning false if unable to fulfil request"""
    try:
        threadNum = int(thread_reference)
    except ValueError:
        threadNum, temp = threadIDandName(mlist, thread_reference)
    return _override_inner(mlist, subscriberID, threadNum, preference)
    
# This is called both by override and override_from_web, above
def _override_inner(mlist, subscriberID, threadID, preference):
    """Add an override entry to the database, returning false if unable to fulfil request"""

    #syslog('info', 'DlistUtils: in override_raw')
    #First check if thread exists
    exists = executeSQL(mlist,
                        "SELECT COUNT(*) FROM thread WHERE thread_id = %d" % threadID)
    if not exists:
        return 0
    
    # Remove any prior override by this user
    executeSQL(mlist,
               "DELETE FROM override WHERE subscriber_id=%s AND thread_id=%s" % (subscriberID, threadID))

    # Now, insert the change
    executeSQL(mlist,
               ["INSERT INTO override (subscriber_id, thread_id, preference)\
                 VALUES (%d, %d, %d)" %
                (subscriberID, threadID, preference)])
    return 1

def overrideURL(list_name, host_name, thread_reference, preference):
    return 'http://%s/mailman/options/%s?override=%s&preference=%d' % (host_name, list_name, thread_reference, preference)
    
def unsubscribeFromList(mlist, key):
    """Indicate that a user has unsubscribed by setting the deleted flag in the subscriber table."""
    executeSQL(mlist,
               ["UPDATE subscriber SET deleted=true WHERE mailman_key='%s'" % key])
    # get the subscriber id from the mailman_key & delete all aliases  
    subscriber_id = getSubscriber_id_raw(mlist, key)
    # Check added 5/16/04
    if subscriber_id == None:
        syslog('error', "DlistUtils.unsubscribeFromList called with '%s', but it can't be found in the SQL database", key)
    else:
        executeSQL(mlist, 
                   ["DELETE FROM alias WHERE subscriber_id='%s'" % subscriber_id])


def subscribeToList(mlist, key):
    """Add a member to the subscriber database, or change the record from deleted if it was already present."""
    # First see if member is subscribed with deleted field = false
    if executeSQL(mlist,
                  ["SELECT COUNT(*) FROM subscriber WHERE mailman_key='%s'"
                   % key]):
        executeSQL(mlist,
                  ["UPDATE subscriber SET deleted=FALSE WHERE mailman_key='%s'"
                   % key])
    else:
        executeSQL(mlist,
                   # format is text-only (1) by default
                   ["INSERT INTO subscriber (mailman_key, preference, deleted, format, suppress)\
                   VALUES ('%s', 1, FALSE, 1, 0)" % key])

def changePreference(mlist, member, preference):
    """Change a user's default preference for new threads."""
    executeSQL(mlist,
               ["UPDATE subscriber SET preference=%d WHERE mailman_key='%s'" %
                (preference, member)])

def changeFormat(mlist, member, format):
    """Change a user's preferred delivery format (plain text and/or html)"""
    executeSQL(mlist,
               ["UPDATE subscriber SET format=%d WHERE mailman_key='%s'" %
                (format, member)])

def get_format(mlist, subscriber_id):
    return int(executeSQL(mlist, "SELECT format FROM subscriber WHERE subscriber_id=%d" % subscriber_id))

def setDisable(mlist, member, flag):
    """Disable/enable delivery based on mm_cfg.DisableDelivery"""
    oldval = executeSQL(mlist, "SELECT suppress FROM subscriber WHERE mailman_key='%s'" % member)
    if flag:
        newval = oldval | 1          # Disable delivery
    else:
        newval = oldval & ~1          # Enable delivery
    executeSQL(mlist,
               ["UPDATE subscriber SET suppress=%d WHERE mailman_key='%s'" %
                (newval, member)])

def setDigest(mlist, member, flag):
    """Disable/enable delivery based on user digest status"""
    oldval = executeSQL(mlist, "SELECT suppress FROM subscriber WHERE mailman_key='%s'" % member)
    if flag:
        newval = oldval | 2          # Suppress delivery (in favor of digests)
    else:
        newval = oldval & ~2          # Enable delivery (instead of digests)

    executeSQL(mlist,
               ["UPDATE subscriber SET suppress=%d WHERE mailman_key='%s'" %
                (newval, member)])

### We have to watch out for a tricky case (that actually happened):
### User foo@bar.com changes her address to something already in the
### subscriber database (possibly deleted).
def changeAddress(mlist, oldaddr, newaddr):
    """Change email address in SQL database"""
    #syslog('info', "Changing email address on %s from '%s' to '%s'", mlist.internal_name(), oldaddr, newaddr)
    ## Check if newaddr is in sql database
    num_matches = executeSQL(mlist, ["SELECT COUNT(*) FROM subscriber WHERE mailman_key = '%s'" % newaddr])
    if num_matches > 1:
        #syslog('error', 'Multiple users with same key (%s): %s', mlist.internal_name(), newaddr)
        return
    if num_matches == 1:
        mergeSubscribers(mlist, oldaddr, newaddr)
    executeSQL(mlist, "UPDATE subscriber SET mailman_key = '%s' WHERE mailman_key = '%s'" % (newaddr, oldaddr))

def mergeSubscribers(mlist, oldaddr, newaddr):
    new_id = executeSQL(mlist, ["SELECT subscriber_id FROM subscriber WHERE mailman_key = '%s'" % oldaddr])
    obsolete_id = executeSQL(mlist, ["SELECT subscriber_id FROM subscriber WHERE mailman_key = '%s'" % newaddr])
    executeSQL(mlist,
               [("UPDATE message SET sender_id = %d WHERE sender_id = %d" % (new_id, obsolete_id)),
                ("UPDATE override SET subscriber_id = %d WHERE subscriber_id = %d" % (new_id, obsolete_id)),
                ("UPDATE alias SET subscriber_id = %d WHERE subscriber_id = %d" % (new_id, obsolete_id)),
                ("DELETE from subscriber WHERE subscriber_id = %d" % obsolete_id)])
    
    
def enabled(mlist):
    """Check whether a given mailing list has dynamic sublists enabled"""
    try:
        return mlist.dlists 
    except:
        return 0

def setFooterText(mlist, msg, msgdata, preference):
    msgdata['dlists_preference'] = preference
    thread_name = msgdata['thread_name']
    thread_id = msgdata['thread_id']
    web_addr = overrideURL(mlist.internal_name(), mlist.host_name, thread_id, not preference)
    if preference == 1:
        subscribe_string = "unsubscribe"
        preposition = "from"
    else:
        subscribe_string = "subscribe"
        preposition = "to"
    email_addr = '%s+%s+%s@%s' % (mlist.internal_name(), thread_name, subscribe_string, mlist.host_name)
    syslog('info', "msg['Subject'] = /%s/", msg['Subject'])
    subject = urllib.quote(msg['Subject'].encode())
    post_addr = "%s+%s@%s" % (mlist.internal_name(), thread_name, mlist.host_name)
    post_addr_with_subject = "%s?Subject=%s" % (post_addr, subject)
    syslog('info', 'post_addr_with_subject = %s', post_addr)

    # Used in Handlers/ToDigest.py
    msgdata['contribute'] = "To contribute to this conversation, send " \
        "your message to <%s+%s@%s>\n" % \
        (mlist.internal_name(), msgdata['thread_name'], mlist.host_name)

    # Used in ToArchive
    msgdata['contribute-html'] = 'To contribute to this conversation, send your message to <a href="mailto:%s">%s</a>\n' % (post_addr, post_addr)

    msgdata['footer-text'] = '\n\nTo %s %s this conversation, send email to <%s> or visit <%s>\nTo contribute to this conversation, use your mailer\'s reply-all or reply-group command or send your message to %s\nTo start a new conversation, send email to <%s+new@%s>\nTo unsubscribe entirely from %s, send email to <%s-request@%s> with subject unsubscribe.' % (subscribe_string, preposition, email_addr, web_addr, post_addr, mlist.internal_name(), mlist.host_name, mlist.internal_name(), mlist.internal_name(), mlist.host_name)
    syslog('info', 'footer-text = /%s/', msgdata['footer-text'])
    msgdata['footer-html'] = '<br>To %s %s this conversation, send email to <a href="mailto:%s">%s</a> or visit <a href="%s">%s</a>.<br>To contribute to this conversation, use your mailer\'s reply-all or reply-group command or send your message to <a href="mailto:%s?subject=%s">%s</a>.<br>To start a new conversation, send email to <a href="mailto:%s+new@%s">%s+new@%s</a><br>To unsubscribe entirely from %s, send mail to <a href="mailto:%s-request@%s?subject=unsubscribe">%s-request@%s</a> with subject unsubscribe.' % (subscribe_string, preposition, email_addr, email_addr, web_addr, web_addr, post_addr, subject, post_addr, mlist.internal_name(), mlist.host_name, mlist.internal_name(), mlist.host_name, mlist.internal_name(), mlist.internal_name(), mlist.host_name, mlist.internal_name(), mlist.host_name)

def canonicalize_sender(mlist, aliases):
    for alias in aliases:
        #syslog('info', 'Checking if <%s> is an alias', alias)
        match = executeSQLmemberQuery(mlist,
                                      "SELECT s.mailman_key FROM subscriber s, alias a WHERE a.pseudonym='%s' AND s.subscriber_id = a.subscriber_id" % alias)
        if len(match) == 1:
            # I should really confirm that there is only one match
            syslog('info', 'Match: %s', match)
            syslog('info', 'Match found: <%s>', match[0])
            return match[0]
        elif len(match) > 1:
            raise DatabaseIntegrityError
        
def get_aliases(mlist, subscriber_id):
    return executeSQLreturningList(mlist, "SELECT  pseudonym FROM alias WHERE subscriber_id=%d" % subscriber_id) 
        
def change_aliases(mlist, subscriber_id, oldAliasList, newAliasList):
    sql = []
    for a in oldAliasList:
        if a not in newAliasList:
            sql.append("DELETE FROM alias WHERE pseudonym='%s'" % a)
    for a in newAliasList:
        if a not in oldAliasList:
            sql.append("INSERT INTO alias (pseudonym, subscriber_id) VALUES ('%s', %d)" % (a, subscriber_id))
    executeSQL(mlist, sql)
    
